@@ -31,6 +31,12 @@
 #include <IOKit/firewire/IOFireWireUnit.h>
 #include <IOKit/IOService.h>
 
+enum
+{
+    kIOFWSBP2FailsOnAckBusy = (1 << 0),
+	kIOFWSBP2FailsOnBusResetsDuringIO = (1 << 1)
+};
+
 /*!
     @class IOFireWireSBP2Target
     @abstract Serves as bridge between IOFireWireUnit and IOFireWireLUN.
@@ -56,7 +62,8 @@ protected:
         kFirmwareRevKey	 			= 0x3c,
         kLUNKey						= 0x14,
         kLUNDirectoryKey			= 0xd4,
-        kManagementAgentOffsetKey	= 0x54
+        kManagementAgentOffsetKey	= 0x54,
+		kUnitCharacteristicsKey 	= 0x3A,
     };
 
     // reserved for future use
@@ -69,7 +76,10 @@ protected:
     bool 				fOpenFromTarget;
     UInt32				fOpenFromLUNCount;    
     IOFireWireUnit * 	fProviderUnit;
-
+    UInt32				fFlags;
+    
+	IOFireWireController * fControl;
+	
 	/////////////////////////////////////////
 	// private internals
 	
@@ -84,7 +94,9 @@ protected:
 												UInt32 softwareRev,
 												UInt32 firmwareRev, 
 												UInt32 lun, 
-												UInt32 devType );
+												UInt32 devType,
+												UInt32 unitCharacteristics,
+												UInt32 managementOffset );
 
 public:
 
@@ -157,6 +169,17 @@ public:
 	
 	virtual bool matchPropertyTable( OSDictionary * table );
 
+    virtual void setTargetFlags( UInt32 flags );
+    virtual UInt32 getTargetFlags( void );
+
+protected:
+    virtual void configurePhysicalFilter( void );
+
+public:
+    virtual void clearTargetFlags( UInt32 flags );
+	virtual IOReturn beginIOCriticalSection( void );
+	virtual void endIOCriticalSection( void );
+
 protected:
 
     OSMetaClassDeclareReservedUnused(IOFireWireSBP2Target, 0);
